@@ -1,13 +1,14 @@
 from __future__ import annotations
 
 import json
-from dataclasses import asdict, is_dataclass
-from typing import Any, Iterable, Iterator, Sequence
+from collections.abc import Iterable, Iterator, Sequence
+from typing import Any
 
 from loguru import logger
 from tqdm import tqdm
 
 from flexeval.core.utils.chat_util import find_first_turn_for_response
+from flexeval.core.utils.json_util import Base64TruncatingJSONEncoder
 
 from .chat_dataset import ChatInstance
 from .few_shot_generator import FewShotGenerator
@@ -82,13 +83,6 @@ def _find_response_context_index(incomplete_messages: list[dict[str, Any]]) -> i
     return None
 
 
-def dataclass_default(o: Any) -> Any:  # noqa: ANN401
-    if is_dataclass(o):
-        return asdict(o)
-    msg = f"Object of type {type(o).__name__} is not JSON serializable"
-    raise TypeError(msg)
-
-
 def execute_conversation_flow(
     language_model: LanguageModel, eval_instances: Sequence[ChatInstance], batch_size: int, gen_kwargs: dict[str, Any]
 ) -> Iterator[dict[str, Any]]:
@@ -161,9 +155,8 @@ def evaluate_chat_response(  # noqa: C901
 ) -> tuple[dict[str, float], list[dict[str, Any]]]:
     logger.info(f"Evaluate the model with gen_kwargs: {gen_kwargs}")
 
-    eval_instances: Sequence[ChatInstance] = eval_dataset
-    if max_instances is not None:
-        eval_instances = [eval_dataset[i] for i in range(min(max_instances, len(eval_instances)))]
+    max_instances = len(eval_dataset) if max_instances is None else min(max_instances, len(eval_dataset))
+    eval_instances: list[ChatInstance] = [eval_dataset[i] for i in range(max_instances)]
 
     if few_shot_generator:
         for eval_instance in eval_instances:
@@ -184,7 +177,7 @@ def evaluate_chat_response(  # noqa: C901
     ):
         if i == 0:
             logger.info("Example of the output")
-            logger.info(json.dumps(output, ensure_ascii=False, indent=4, default=dataclass_default))
+            logger.info(json.dumps(output, ensure_ascii=False, indent=4, cls=Base64TruncatingJSONEncoder))
         outputs.append(output)
 
     language_model.cleanup_resources()
@@ -232,6 +225,8 @@ def evaluate_chat_response(  # noqa: C901
             "references": output["chat_instance"].references,
             **output["metrics"],
         }
+        if output["lm_output"].raw_text:
+            restructured_output["raw_lm_output"] = output["lm_output"].raw_text
         restructured_outputs.append(restructured_output)
 
     return metrics_summary_dict, restructured_outputs
