@@ -1,5 +1,7 @@
-from .base import LanguageModel
+from .base import LanguageModel, LMOutput
 from .hf_lm import HuggingFaceLM
+from .litellm_api import LiteLLMChatAPI
 from .openai_api import OpenAIChatAPI, OpenAICompletionAPI
 from .openai_batch_api import OpenAIChatBatchAPI
 from .vllm_model import VLLM
+from .vllm_serve_lm import VLLMServeLM
