@@ -1,21 +1,21 @@
 from __future__ import annotations
 
-import asyncio
 import itertools
-from typing import Any, Awaitable, Callable, TypeVar
+import os
+import time
+from concurrent.futures import ThreadPoolExecutor, as_completed
+from typing import Any, Callable, TypeVar
 
 import openai
 import tiktoken
 from loguru import logger
-from openai import AsyncOpenAI, BaseModel
-from openai._types import NotGiven
+from openai import BaseModel, OpenAI
 from openai.types.chat import ChatCompletion, ChatCompletionMessage
 from openai.types.chat.chat_completion import Choice
 
+from flexeval.core.language_model.base import LanguageModel, LMOutput, normalize_stop_sequences
 from flexeval.core.string_processor import StringProcessor
 
-from .base import LanguageModel, LMOutput, normalize_stop_sequences
-
 T = TypeVar("T")
 
 
@@ -40,24 +40,28 @@ EMPTY_RESPONSE = ChatCompletion(
 )
 
 
-async def _retry_on_error(
-    openai_call: Callable[[], Awaitable[T]],
+def _retry_on_error(
+    openai_call: Callable[[], T],
     empty_response: BaseModel,
-    max_num_trials: int = 5,
-    first_wait_time: int = 10,
-) -> Awaitable[T]:
+    max_num_trials: int | None = None,
+    first_wait_time: int | None = None,
+    max_wait_time: int | None = None,
+) -> T:
+    max_num_trials = max_num_trials or 5
+    first_wait_time = first_wait_time or 10
+    max_wait_time = max_wait_time or 80
     for i in range(max_num_trials):
         try:
-            return await openai_call()
+            return openai_call()
         except openai.APIError as e:  # noqa: PERF203
             if i == max_num_trials - 1:
                 # Since reaching maximum number of trials, exit for-loop and return
                 # empty response.
                 break
             logger.warning(f"We got an error: {e}")
-            wait_time_seconds = first_wait_time * (2**i)
+            wait_time_seconds = min(max_wait_time, first_wait_time * (2**i))
             logger.warning(f"Wait for {wait_time_seconds} seconds...")
-            await asyncio.sleep(wait_time_seconds)
+            time.sleep(wait_time_seconds)
 
     logger.warning(f"We reached maximum number of trials ({max_num_trials} trials.).")
     logger.warning("Response including empty string is returned.")
@@ -78,6 +82,8 @@ class OpenAIChatAPI(LanguageModel):
         string_processors: A single or a list of StringProcessor objects to process the model's output.
         model_limit_new_tokens: An upper limit on the number of tokens the model can generate.
             For example, if a too-large `max_new_tokens` is given to generate_chat_response(), this value will cap it.
+        max_parallel_requests: Maximum number of parallel requests to send to the OpenAI API.
+        tools: Default tools to use in chat responses when no tools are explicitly provided.
     """
 
     def __init__(
@@ -88,12 +94,17 @@ class OpenAIChatAPI(LanguageModel):
         developer_message: str | None = None,
         string_processors: StringProcessor | list[StringProcessor] | None = None,
         model_limit_new_tokens: int | None = None,
+        max_parallel_requests: int | None = None,
+        tools: list[dict[str, Any]] | None = None,
+        max_num_trials: int | None = None,
+        first_wait_time: int | None = None,
+        max_wait_time: int | None = None,
     ) -> None:
-        super().__init__(string_processors=string_processors)
+        super().__init__(string_processors=string_processors, tools=tools)
         self.model = model
         if api_headers is None:
             api_headers = {}
-        client = AsyncOpenAI(**api_headers)
+        client = OpenAI(**api_headers)
         self.api_call_func = client.chat.completions.create
         self.empty_response = EMPTY_RESPONSE
         self.default_gen_kwargs = default_gen_kwargs or {}
@@ -103,8 +114,12 @@ class OpenAIChatAPI(LanguageModel):
 
         self.developer_message = developer_message
         self.model_limit_new_tokens = model_limit_new_tokens
+        self.max_parallel_requests = max_parallel_requests
+        self.max_num_trials = max_num_trials
+        self.first_wait_time = first_wait_time
+        self.max_wait_time = max_wait_time
 
-    async def _async_batch_run_chatgpt(
+    def _parallel_run_chatgpt(
         self,
         messages_list: list[list[dict[str, Any]]],
         tools_list: list[list[dict[str, Any]] | None] | None = None,
@@ -150,22 +165,38 @@ class OpenAIChatAPI(LanguageModel):
         if tools_list is None:
             tools_list = [None] * len(messages_list)
 
-        tasks = [
-            _retry_on_error(
-                # Define an anonymous function with a lambda expression and pass it,
-                # and call it inside the _retry_on_error function
-                openai_call=lambda messages=messages, tools=tools: self.api_call_func(
-                    model=self.model,
-                    messages=messages,
-                    tools=tools,
-                    stop=stop_sequences or NotGiven(),
-                    **gen_kwargs,
-                ),
-                empty_response=self.empty_response,
-            )
-            for messages, tools in zip(messages_list, tools_list)
-        ]
-        return await asyncio.gather(*tasks)
+        total = len(messages_list)
+        # Progress logging controlled via the environment variable OPENAI_PROGRESS_EVERY_N
+        # N>0: log every N completions; 0 or unset: disables
+        prog_every_n = int(os.getenv("OPENAI_PROGRESS_EVERY_N", "0"))
+        max_workers = self.max_parallel_requests or total
+        with ThreadPoolExecutor(max_workers=max_workers) as executor:
+            future_to_idx = {}
+            for idx, (messages, tools) in enumerate(zip(messages_list, tools_list)):
+                future = executor.submit(
+                    _retry_on_error,
+                    openai_call=lambda messages=messages, tools=tools: self.api_call_func(
+                        **{
+                            **({"model": self.model, "messages": messages} | gen_kwargs),
+                            **({"tools": tools} if tools else {}),
+                            **({"stop": stop_sequences} if stop_sequences else {}),
+                        }
+                    ),
+                    empty_response=self.empty_response,
+                    max_num_trials=self.max_num_trials,
+                    first_wait_time=self.first_wait_time,
+                    max_wait_time=self.max_wait_time,
+                )
+                future_to_idx[future] = idx
+
+            results: list[ChatCompletion] = [self.empty_response] * total
+            for done_count, future in enumerate(as_completed(future_to_idx.keys()), start=1):
+                idx = future_to_idx[future]
+                results[idx] = future.result()
+                if prog_every_n and (done_count % prog_every_n == 0 or done_count == total):
+                    logger.info(f"[progress] {done_count}/{total} ({done_count/total:.1%}) done")
+
+            return results
 
     def _batch_complete_text(
         self,
@@ -175,13 +206,11 @@ class OpenAIChatAPI(LanguageModel):
         **kwargs,
     ) -> list[LMOutput]:
         messages_list = [[{"role": "user", "content": text}] for text in text_list]
-        api_responses = asyncio.run(
-            self._async_batch_run_chatgpt(
-                messages_list,
-                stop_sequences=stop_sequences,
-                max_new_tokens=max_new_tokens,
-                **kwargs,
-            ),
+        api_responses = self._parallel_run_chatgpt(
+            messages_list,
+            stop_sequences=stop_sequences,
+            max_new_tokens=max_new_tokens,
+            **kwargs,
         )
         outputs = [
             LMOutput(text=res.choices[0].message.content, finish_reason=res.choices[0].finish_reason)
@@ -198,9 +227,7 @@ class OpenAIChatAPI(LanguageModel):
         tools_list: list[list[dict[str, Any]] | None] | None = None,
         **kwargs,
     ) -> list[LMOutput]:
-        api_responses = asyncio.run(
-            self._async_batch_run_chatgpt(chat_messages_list, tools_list=tools_list, **kwargs),
-        )
+        api_responses = self._parallel_run_chatgpt(chat_messages_list, tools_list=tools_list, **kwargs)
         outputs = [
             LMOutput(
                 text=res.choices[0].message.content,
@@ -244,14 +271,12 @@ class OpenAIChatAPI(LanguageModel):
 
         # For saving cost, remove duplication from message_list for an API request.
         unique_prompt_list = remove_duplicates_from_prompt_list(prompt_list)
-        api_responses = asyncio.run(
-            self._async_batch_run_chatgpt(
-                unique_prompt_list,
-                max_completion_tokens=1,
-                seed=seed,
-                logprobs=True,
-                top_logprobs=top_logprobs,
-            ),
+        api_responses = self._parallel_run_chatgpt(
+            unique_prompt_list,
+            max_completion_tokens=1,
+            seed=seed,
+            logprobs=True,
+            top_logprobs=top_logprobs,
         )
 
         log_probs = []
@@ -323,6 +348,7 @@ class OpenAICompletionAPI(LanguageModel):
         api_headers: A dictionary of headers to use when making requests to the OpenAI API.
         default_gen_kwargs: Default generation kwargs to use when calling the API.
         string_processors: A single or a list of StringProcessor objects to process the model's output.
+        max_parallel_requests: Maximum number of parallel requests to send to the OpenAI API.
     """
 
     def __init__(
@@ -331,20 +357,22 @@ class OpenAICompletionAPI(LanguageModel):
         api_headers: dict[str, str] | None = None,
         default_gen_kwargs: dict[str, Any] | None = None,
         string_processors: StringProcessor | list[StringProcessor] | None = None,
+        max_parallel_requests: int | None = None,
     ) -> None:
         super().__init__(string_processors=string_processors)
         self.model = model
         if api_headers is None:
             api_headers = {}
-        client = AsyncOpenAI(**api_headers)
+        client = OpenAI(**api_headers)
         self.api_call_func = client.completions.create
         self.empty_response = EMPTY_RESPONSE
         self.default_gen_kwargs = default_gen_kwargs or {}
+        self.max_parallel_requests = max_parallel_requests
         # convert the flexeval-specific argument name to the OpenAI-specific name
         if "max_new_tokens" in self.default_gen_kwargs:
             self.default_gen_kwargs["max_tokens"] = self.default_gen_kwargs.pop("max_new_tokens")
 
-    async def _async_batch_run_completion(
+    def _parallel_run_chatgpt(
         self,
         prompt_list: list[str],
         stop_sequences: str | list[str] | None = None,
@@ -368,20 +396,23 @@ class OpenAICompletionAPI(LanguageModel):
         if stop_sequences:
             gen_kwargs["stop"] = stop_sequences
 
-        tasks = [
-            _retry_on_error(
-                # Define an anonymous function with a lambda expression and pass it,
-                # and call it inside the _retry_on_error function
-                openai_call=lambda x=ms: self.api_call_func(
-                    model=self.model,
-                    prompt=x,
-                    **gen_kwargs,
-                ),
-                empty_response=self.empty_response,
-            )
-            for ms in prompt_list
-        ]
-        return await asyncio.gather(*tasks)
+        max_workers = self.max_parallel_requests or len(prompt_list)
+        with ThreadPoolExecutor(max_workers=max_workers) as executor:
+            futures = [
+                executor.submit(
+                    _retry_on_error,
+                    # Define an anonymous function with a lambda expression and pass it,
+                    # and call it inside the _retry_on_error function
+                    openai_call=lambda x=ms: self.api_call_func(
+                        model=self.model,
+                        prompt=x,
+                        **gen_kwargs,
+                    ),
+                    empty_response=self.empty_response,
+                )
+                for ms in prompt_list
+            ]
+        return [future.result() for future in futures]
 
     def _batch_complete_text(
         self,
@@ -390,13 +421,11 @@ class OpenAICompletionAPI(LanguageModel):
         max_new_tokens: int | None = None,
         **kwargs,
     ) -> list[LMOutput]:
-        api_responses = asyncio.run(
-            self._async_batch_run_completion(
-                text_list,
-                stop_sequences=stop_sequences,
-                max_new_tokens=max_new_tokens,
-                **kwargs,
-            ),
+        api_responses = self._parallel_run_chatgpt(
+            text_list,
+            stop_sequences=stop_sequences,
+            max_new_tokens=max_new_tokens,
+            **kwargs,
         )
 
         return [LMOutput(text=res.choices[0].text, finish_reason=res.choices[0].finish_reason) for res in api_responses]
