@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import os
 from typing import Any
 
 from flexeval.core.utils.jinja2_utils import JINJA2_ENV
@@ -7,6 +8,13 @@ from flexeval.core.utils.jinja2_utils import JINJA2_ENV
 from .base import PromptTemplate
 
 
+def instantiate_prompt_template_from_string(template_or_path: str) -> Jinja2PromptTemplate:
+    # Use `os.path.isfile` instead of `Path.is_file()` to avoid "OSError: [Errno 36] File name too long"
+    if os.path.isfile(template_or_path):  # noqa: PTH113
+        return Jinja2PromptTemplate(template_path=template_or_path)
+    return Jinja2PromptTemplate(template=template_or_path)
+
+
 class Jinja2PromptTemplate(PromptTemplate):
     """
     Embed task inputs using Jinja2 template engine.
